@@ -1,5 +1,5 @@
 # engine/default.py
-# Copyright (C) 2005-2020 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2021 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
@@ -114,12 +114,11 @@ class DefaultDialect(interfaces.Dialect):
     max_identifier_length = 9999
     _user_defined_max_identifier_length = None
 
-    # length at which to truncate
-    # the name of an index.
-    # Usually None to indicate
-    # 'use max_identifier_length'.
-    # thanks to MySQL, sigh
+    # sub-categories of max_identifier_length.
+    # currently these accommodate for MySQL which allows alias names
+    # of 255 but DDL names only of 64.
     max_index_name_length = None
+    max_constraint_name_length = None
 
     supports_sane_rowcount = True
     supports_sane_multi_rowcount = True
@@ -129,8 +128,13 @@ class DefaultDialect(interfaces.Dialect):
     supports_empty_insert = True
     supports_multivalues_insert = False
 
+    supports_is_distinct_from = True
+
     supports_server_side_cursors = False
 
+    # extra record-level locking features (#4860)
+    supports_for_update_of = False
+
     server_version_info = None
 
     construct_arguments = None
@@ -191,7 +195,7 @@ class DefaultDialect(interfaces.Dialect):
     @util.deprecated_params(
         convert_unicode=(
             "1.3",
-            "The :paramref:`.create_engine.convert_unicode` parameter "
+            "The :paramref:`_sa.create_engine.convert_unicode` parameter "
             "and corresponding dialect-level parameters are deprecated, "
             "and will be removed in a future release.  Modern DBAPIs support "
             "Python Unicode natively and this parameter is unnecessary.",
@@ -284,8 +288,8 @@ class DefaultDialect(interfaces.Dialect):
         """True if this dialect supports sane rowcount even if RETURNING is
         in use.
 
-        For dialects that don't support RETURNING, this is synomous
-        with supports_sane_rowcount.
+        For dialects that don't support RETURNING, this is synonymous with
+        ``supports_sane_rowcount``.
 
         """
         return self.supports_sane_rowcount
@@ -294,6 +298,14 @@ class DefaultDialect(interfaces.Dialect):
     def get_pool_class(cls, url):
         return getattr(cls, "poolclass", pool.QueuePool)
 
+    @classmethod
+    def load_provisioning(cls):
+        package = ".".join(cls.__module__.split(".")[0:-1])
+        try:
+            __import__(package + ".provision")
+        except ImportError:
+            pass
+
     def initialize(self, connection):
         try:
             self.server_version_info = self._get_server_version_info(
@@ -309,7 +321,7 @@ class DefaultDialect(interfaces.Dialect):
             self.default_schema_name = None
 
         try:
-            self.default_isolation_level = self.get_isolation_level(
+            self.default_isolation_level = self.get_default_isolation_level(
                 connection.connection
             )
         except NotImplementedError:
@@ -339,17 +351,7 @@ class DefaultDialect(interfaces.Dialect):
             )
 
     def on_connect(self):
-        """return a callable which sets up a newly created DBAPI connection.
-
-        This is used to set dialect-wide per-connection options such as
-        isolation modes, unicode modes, etc.
-
-        If a callable is returned, it will be assembled into a pool listener
-        that receives the direct DBAPI connection, with all wrappers removed.
-
-        If None is returned, no listener will be generated.
-
-        """
+        # inherits the docstring from interfaces.Dialect.on_connect
         return None
 
     def _check_max_identifier_length(self, connection):
@@ -364,6 +366,22 @@ class DefaultDialect(interfaces.Dialect):
         """
         return None
 
+    def get_default_isolation_level(self, dbapi_conn):
+        """Given a DBAPI connection, return its isolation level, or
+        a default isolation level if one cannot be retrieved.
+
+        May be overridden by subclasses in order to provide a
+        "fallback" isolation level for databases that cannot reliably
+        retrieve the actual isolation level.
+
+        By default, calls the :meth:`_engine.Interfaces.get_isolation_level`
+        method, propagating any exceptions raised.
+
+        .. versionadded:: 1.3.22
+
+        """
+        return self.get_isolation_level(dbapi_conn)
+
     def _check_unicode_returns(self, connection, additional_tests=None):
         if util.py2k and not self.supports_unicode_statements:
             cast_to = util.binary_type
@@ -448,7 +466,7 @@ class DefaultDialect(interfaces.Dialect):
 
         This method looks for a dictionary called
         ``colspecs`` as a class or instance-level variable,
-        and passes on to :func:`.types.adapt_type`.
+        and passes on to :func:`_types.adapt_type`.
 
         """
         return sqltypes.adapt_type(typeobj, self.colspecs)
@@ -486,9 +504,11 @@ class DefaultDialect(interfaces.Dialect):
             )
 
     def connect(self, *cargs, **cparams):
+        # inherits the docstring from interfaces.Dialect.connect
         return self.dbapi.connect(*cargs, **cparams)
 
     def create_connect_args(self, url):
+        # inherits the docstring from interfaces.Dialect.create_connect_args
         opts = url.translate_connect_args()
         opts.update(url.query)
         return [[], opts]
@@ -651,6 +671,26 @@ class DefaultDialect(interfaces.Dialect):
         return name
 
 
+class _RendersLiteral(object):
+    def literal_processor(self, dialect):
+        def process(value):
+            return "'%s'" % value
+
+        return process
+
+
+class _StrDateTime(_RendersLiteral, sqltypes.DateTime):
+    pass
+
+
+class _StrDate(_RendersLiteral, sqltypes.Date):
+    pass
+
+
+class _StrTime(_RendersLiteral, sqltypes.Time):
+    pass
+
+
 class StrCompileDialect(DefaultDialect):
 
     statement_compiler = compiler.StrSQLCompiler
@@ -667,6 +707,12 @@ class StrCompileDialect(DefaultDialect):
 
     supports_simple_order_by_label = True
 
+    colspecs = {
+        sqltypes.DateTime: _StrDateTime,
+        sqltypes.Date: _StrDate,
+        sqltypes.Time: _StrTime,
+    }
+
 
 class DefaultExecutionContext(interfaces.ExecutionContext):
     isinsert = False
@@ -1407,12 +1453,13 @@ class DefaultExecutionContext(interfaces.ExecutionContext):
     generation function, e.g. as described at :ref:`context_default_functions`.
     It consists of a dictionary which includes entries for each column/value
     pair that is to be part of the INSERT or UPDATE statement. The keys of the
-    dictionary will be the key value of each :class:`.Column`, which is usually
+    dictionary will be the key value of each :class:`_schema.Column`,
+    which is usually
     synonymous with the name.
 
     Note that the :attr:`.DefaultExecutionContext.current_parameters` attribute
     does not accommodate for the "multi-values" feature of the
-    :meth:`.Insert.values` method.  The
+    :meth:`_expression.Insert.values` method.  The
     :meth:`.DefaultExecutionContext.get_current_parameters` method should be
     preferred.
 
@@ -1432,11 +1479,13 @@ class DefaultExecutionContext(interfaces.ExecutionContext):
         :ref:`context_default_functions`. When invoked, a dictionary is
         returned which includes entries for each column/value pair that is part
         of the INSERT or UPDATE statement. The keys of the dictionary will be
-        the key value of each :class:`.Column`, which is usually synonymous
+        the key value of each :class:`_schema.Column`,
+        which is usually synonymous
         with the name.
 
         :param isolate_multiinsert_groups=True: indicates that multi-valued
-         INSERT constructs created using :meth:`.Insert.values` should be
+         INSERT constructs created using :meth:`_expression.Insert.values`
+         should be
          handled by returning only the subset of parameters that are local
          to the current column default invocation.   When ``False``, the
          raw parameters of the statement are returned including the
